#  tests for cvxopt-1.2.7-py38he924ae6_1 (this is a generated file);
print('===== testing package: cvxopt-1.2.7-py38he924ae6_1 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
#!/usr/bin/env python
# -*- coding: utf-8 -*-
if __name__ == '__main__':
    import sys
    import py.test

    sys.exit(py.test.main())
#  --- run_test.py (end) ---

print('===== cvxopt-1.2.7-py38he924ae6_1 OK =====');
print("import: 'cvxopt'")
import cvxopt

print("import: 'cvxopt.base'")
import cvxopt.base

print("import: 'cvxopt.blas'")
import cvxopt.blas

print("import: 'cvxopt.cholmod'")
import cvxopt.cholmod

print("import: 'cvxopt.lapack'")
import cvxopt.lapack

print("import: 'cvxopt.misc_solvers'")
import cvxopt.misc_solvers

print("import: 'cvxopt.umfpack'")
import cvxopt.umfpack

print("import: 'cvxopt.glpk'")
import cvxopt.glpk

print("import: 'cvxopt.fftw'")
import cvxopt.fftw

